package org.artifactory.queue.execution;

import lombok.NonNull;
import org.artifactory.api.repo.WorkItem;
import org.artifactory.queue.services.PersistentQueue;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;

/**
 * Passing PersistentQueue trigger through the work queue to allow control of the concurrent running queues number.
 * using the ConstantValues.persistentQueueTriggerNumberOfThread value.
 *
 * @author avishayh
 */
public class PersistentQueueTriggerWorkItem extends WorkItem {

    private final PersistentQueue queue;

    public PersistentQueueTriggerWorkItem(@NonNull PersistentQueue queue) {
        this.queue = queue;
    }

    public PersistentQueue getQueue() {
        return queue;
    }

    @NotNull
    @Override
    public String getUniqueKey() {
        return queue.getName();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) { return true; }
        if (o == null || getClass() != o.getClass()) { return false; }
        PersistentQueueTriggerWorkItem that = (PersistentQueueTriggerWorkItem) o;
        return Objects.equals(queue.getName(), that.queue.getName());
    }

    @Override
    public int hashCode() {
        return Objects.hash(queue.getName());
    }
}
