/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.descriptor.repo;

import org.mockito.Mockito;
import org.testng.annotations.Test;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertNotEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;

/**
 * Tests the RemoteRepoDescriptor class.
 *
 * @author Yossi Shaul
 */
@Test
public class RemoteRepoDescriptorTest {
    public void defaultConstructor() {
        RemoteRepoDescriptor remote = new RemoteRepoDescriptor() {
        };
        assertNull(remote.getKey());
        assertEquals(remote.getIncludesPattern(), "**/*");
        assertNull(remote.getExcludesPattern());
        assertNull(remote.getDescription());
        assertNull(remote.getRepoLayout());
        assertEquals(remote.getAssumedOfflinePeriodSecs(), 300);
        assertEquals(remote.getMaxUniqueSnapshots(), 0);
        assertEquals(remote.getMaxUniqueTags(), 0);
        assertEquals(remote.getMissedRetrievalCachePeriodSecs(), 1800);
        assertEquals(remote.getRetrievalCachePeriodSecs(), 7200);
        assertEquals(remote.getChecksumPolicyType(), ChecksumPolicyType.GEN_IF_ABSENT);
        assertNull(remote.getUrl());
        assertNull(remote.getRemoteRepoLayout());
        assertFalse(remote.isOffline());
        assertFalse(remote.isBlackedOut());
        assertFalse(remote.isCache());
        assertFalse(remote.isLocal());
        assertFalse(remote.isHardFail());
        assertTrue(remote.isStoreArtifactsLocally());
        assertFalse(remote.isFetchJarsEagerly());
        assertFalse(remote.isFetchSourcesEagerly());
        assertTrue(remote.isSuppressPomConsistencyChecks(),
                "Default should not supress pom consistency checks");
        assertEquals(remote.getUnusedArtifactsCleanupPeriodHours(), 0);
        assertFalse(remote.isShareConfiguration());
        assertNotNull(remote.getPropertySets(), "Property sets list should not be null");
        assertFalse(remote.isRejectInvalidJars());
        assertNotEquals(remote.getType(), RepoType.NuGet, "NuGet should be off by default.");
        assertEquals(remote.getDockerApiVersion(), DockerApiVersion.V2, "Docker API version should be V2 by default.");
    }

    public void shouldCallEmptyProcessUrlForNonHttpRepoDescriptor() {
        final RemoteRepoDescriptor remoteRepoDescriptor = Mockito.spy(RemoteRepoDescriptor.class);
        remoteRepoDescriptor.processUrl();
        Mockito.verify(remoteRepoDescriptor, Mockito.times(0)).setUrl(any());
    }

    public void shouldProcessUrlForHttpRepoDescriptor() {
        final RemoteRepoDescriptor remoteRepoDescriptor = Mockito.spy(HttpRepoDescriptor.class);
        when(remoteRepoDescriptor.getUrl()).thenReturn("https://admin:password@localhost:8080/conan-smart-remote");
        remoteRepoDescriptor.processUrl();
        Mockito.verify(remoteRepoDescriptor).processUrl();
        Mockito.verify(remoteRepoDescriptor, Mockito.times(1)).setUrl(any());
    }

}
