package org.artifactory.environment.converter.local.version.v7;

import org.artifactory.common.ConstantValues;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.version.ArtifactoryVersionProvider;
import org.artifactory.version.CompoundVersionDetails;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

/**
 * @author Tamir Hadad
 */
@Test
public class V700UpgradePrerequisiteVerifierTest extends ArtifactoryHomeBoundTest {

    private V700UpgradePrerequisiteVerifier verifier;

    @BeforeClass
    public void init() {
        verifier = new V700UpgradePrerequisiteVerifier();
    }

    public void testInterested() {
        CompoundVersionDetails source = mock(CompoundVersionDetails.class);
        when(source.getVersion()).thenReturn(ArtifactoryVersionProvider.v598.get());
        CompoundVersionDetails dest = mock(CompoundVersionDetails.class);
        when(dest.getVersion()).thenReturn(ArtifactoryVersionProvider.v700m001.get());
        assertTrue(verifier.isInterested(homeStub, source, dest));
    }

    public void testNotInterested() {
        CompoundVersionDetails source = mock(CompoundVersionDetails.class);
        when(source.getVersion()).thenReturn(ArtifactoryVersionProvider.v600.get());
        CompoundVersionDetails dest = mock(CompoundVersionDetails.class);
        when(dest.getVersion()).thenReturn(ArtifactoryVersionProvider.v700m001.get());
        assertFalse(verifier.isInterested(homeStub, source, dest));
    }

    @Test(expectedExceptions = IllegalStateException.class)
    public void testBlockConvert() {
        homeStub.setProperty(ConstantValues.allowAnyUpgrade, null);
        CompoundVersionDetails source = mock(CompoundVersionDetails.class);
        CompoundVersionDetails dest = mock(CompoundVersionDetails.class);
        verifier.convert(homeStub, source, dest);
    }

    public void testAllowConvertByProp() {
        homeStub.setProperty(ConstantValues.allowAnyUpgrade, ArtifactoryVersionProvider.v700m001.get().getVersion());
        CompoundVersionDetails source = mock(CompoundVersionDetails.class);
        CompoundVersionDetails dest = mock(CompoundVersionDetails.class);
        verifier.convert(homeStub, source, dest);
    }
}